@@ -2,13 +2,14 @@
 
 /*
  * Remove /./ and /../ and // from a pathname
- * An absolute pathname argument is required. 
+ * An absolute pathname argument is required.
  * Returns NULL on error, otherwise NULL terminated
  * sanitized pathname.
  *
  * Also see realpath(3)
  */
-char * abspath(char *path) {
+char * abspath(char *path)
+{
 	char *p, *c;
 	char *slash, *abspath2;
 	char *abspath;
@@ -20,8 +21,8 @@ char * abspath(char *path) {
 	if (!g_path_is_absolute(path))
 		return NULL;
 
-	/* abspath can be NULL or abspath[0] == '\0'. The NULL 
-	 * is initial. the [0] == '\0' is when we got back to 
+	/* abspath can be NULL or abspath[0] == '\0'. The NULL
+	 * is initial. the [0] == '\0' is when we got back to
 	 * the root
 	 */
 	abspath = NULL;
@@ -55,9 +56,13 @@ char * abspath(char *path) {
 		} else {
 			if (abspath == NULL || abspath[0] == '\0' ||
 				(strcmp(abspath, "/") == 0) )  {
+				g_free(abspath);
 				abspath = g_strconcat("/", p, NULL);
 			} else {
-				abspath = g_strjoin("/", abspath, p, NULL);
+				gchar* tmp = g_strdup(abspath);
+				g_free(abspath);
+				abspath = g_strjoin("/", tmp, p, NULL);
+				g_free(tmp);
 			}
 			*c = '/';
 		}
@@ -66,5 +71,6 @@ char * abspath(char *path) {
 	if (abspath == NULL || abspath[0] == '\0')
 		abspath = g_strdup("/");
 
+	g_free(abspath2);
 	return abspath;
 }
