@@ -1,12 +1,11 @@
-/* 
- * Copyright (c) 2009 Miek Gieben
- * child-tr.c handle child stuff
+/*
+ * Copyright (c) 2009-2011 Miek Gieben
+ * child.c handle child stuff
  */
 
-#include "rdup-tr.h"
+#include "rdup.h"
 
-extern sig_atomic_t sig;
-extern gint opt_verbose;
+extern int sig;
 
 /* signal.c */
 void got_sig(int signal);
@@ -19,15 +18,11 @@ close_pipes(GSList *pipes, int n1, int n2)
 	int *q;
 	int j;
 
-	for (j = 0, p = g_slist_nth(pipes, 0); p; p = p->next, j++) { 
+	for (j = 0, p = g_slist_nth(pipes, 0); p; p = p->next, j++) {
 		q = p->data;
 /*		msg("Pipe set %d fds %d %d", j, q[0], q[1]); */
 		if (j == n1 || j == n2)
 			continue;
-#if 0
-		if (opt_verbose > 0) 
-			msg("Closing pipe set %d", j);
-#endif
 		close(q[0]);
 		close(q[1]);
 	}
@@ -39,30 +34,36 @@ wait_pids(GSList *pids, int flags)
 {
 	GSList *p;
 	int status;
-	int ret = 0;
 
-	if (!pids)
-		return 0;
-
-	for (p = g_slist_nth(pids, 0); p; p = p->next) { 
+	for (p = g_slist_nth(pids, 0); p; p = p->next) {
                 if (sig != 0)
                         signal_abort(sig);
 
-		waitpid(*(pid_t* )(p->data), &status, flags);
-		if (WIFEXITED(status)) {
+#ifdef DEBUG
+		msgd(__func__, __LINE__,"Waiting for pid %d", (int) *(pid_t*)(p->data));
+#endif /* DEBUG */
+
+		/* -1 on error */
+		waitpid(*(pid_t* )(p->data), &status, flags); /* errno ECHILD is ok */
 #if 0
-			msg("Child exit %d\n", WEXITSTATUS(status)); 
-#endif
+		if (WIFEXITED(status)) {
+			/* msg("Child exit %d", WEXITSTATUS(status));
 			if (WEXITSTATUS(status) != 0)
 				ret = -1;
+			*/
+			/* assume ok */
+			ret = 0;
+		} else {
+			ret = -1;
 		}
+#endif
 	}
-	return ret;
+	return 0;
 }
 
 /* create pipes and childs, return pids */
 GSList *
-create_childeren(GSList *child, GSList **pipes, int file) 
+create_childeren(GSList *child, GSList **pipes, int file)
 {
 	GSList  *p;
 	GSList	*pids	= NULL;
@@ -76,13 +77,13 @@ create_childeren(GSList *child, GSList **pipes, int file)
 	if (!child)
 		return NULL;
 
-	/* create all pipes before forking 
+	/* create all pipes before forking
 	 * As a parent we read from the last pipe created
 	 * We attach file to the input of the first child
 	 * This eliminates to use of a tmp file
 	 */
 	childs = g_slist_length(child);
-	for (j = 0; j < childs; j++) { 
+	for (j = 0; j < childs; j++) {
 		pips = g_malloc(2 * sizeof(int));
 		if (pipe(pips) == -1) {
 			msg(_("Failure creating pipes"));
@@ -91,7 +92,7 @@ create_childeren(GSList *child, GSList **pipes, int file)
 		cpipe = g_slist_append(cpipe, pips);
 	}	
 
-	for (j = 0, p = g_slist_nth(child, 0); p; p = p->next, j++) { 
+	for (j = 0, p = g_slist_nth(child, 0); p; p = p->next, j++) {
                 if (sig != 0)
                         signal_abort(sig);
 
@@ -102,7 +103,8 @@ create_childeren(GSList *child, GSList **pipes, int file)
 
 		if ( (*cpid = fork()) == -1) {
 			msg(_("Fork error"));
-			exit(EXIT_FAILURE);
+			return NULL; /* more gracefull then exit */
+			/* exit(EXIT_FAILURE); */
 		}
 
 		if (*cpid != 0) {			/* parent */
@@ -139,8 +141,10 @@ create_childeren(GSList *child, GSList **pipes, int file)
 			}
 
 			/* finally ... exec */
-			if ( execvp(args[0], args) == -1)
+			if ( execvp(args[0], args) == -1) {
+				msg(_("Failed to exec `%s\': %s"), args[0], strerror(errno));
 				exit(EXIT_FAILURE);
+			}
 			
 			/* never reached */
 			exit(EXIT_SUCCESS);
