@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 - 2009 Miek Gieben
+ * Copyright (c) 2005 - 2011 Miek Gieben
  * See LICENSE for the license
  *
  * Directory crawler
@@ -8,26 +8,27 @@
 
 extern gboolean opt_onefilesystem;
 extern gboolean opt_nobackup;
+extern gboolean opt_atime;
 extern time_t opt_timestamp;
 extern gint opt_verbose;
 extern GSList *pregex_list;
 
 /* common.c */
-struct r_entry * entry_dup(struct r_entry *f);
-void entry_free(struct r_entry *f);
+struct rdup * entry_dup(struct rdup *);
+void entry_free(struct rdup *);
 
 /**
  * prepend path leading up to backup directory to the tree
  */
 gboolean
-dir_prepend(GTree *t, char *path)
+dir_prepend(GTree *t, char *path, GHashTable *u, GHashTable *g)
 {
 	char *c;
 	char *p;
 	char *path2;
 	size_t len;
 	struct stat s;
-	struct r_entry e;
+	struct rdup e;
 
 	path2 = g_strdup(path);
 	len   = strlen(path);
@@ -46,10 +47,15 @@ dir_prepend(GTree *t, char *path)
 			return FALSE;
 		}
 		e.f_name      = path2;
+		e.f_target    = NULL;
 		e.f_name_size = strlen(path2);
 		e.f_uid       = s.st_uid;
+		e.f_user      = lookup_user(u, e.f_uid);
 		e.f_gid       = s.st_gid;
+		e.f_group     = lookup_group(g, e.f_gid);
 		e.f_ctime     = s.st_ctime;
+		e.f_mtime     = s.st_mtime;
+		e.f_atime     = s.st_atime;
 		e.f_mode      = s.st_mode;
 		e.f_size      = s.st_size;
 		e.f_dev       = s.st_dev;
@@ -57,9 +63,20 @@ dir_prepend(GTree *t, char *path)
 		e.f_ino       = s.st_ino;
 		e.f_lnk	      = 0;
 
-		/* symlinks; also put the -> name in f_name */
-		if (S_ISLNK(s.st_mode))
-			e = *(sym_link(&e, NULL));
+		/* symlinks; also set the target */
+		if (S_ISLNK(s.st_mode)) {
+			e.f_target = slink(&e);
+			e.f_size = e.f_name_size;
+			e.f_name_size += 4 + strlen(e.f_target);
+			/* When we encounter a symlink on this level, it is very hard to make this
+			 * backup work, because the target may fall out of the backup. If this
+			 * is the case the entire backup fails. Gnu tar only show the symlink
+			 * and then stops. We do now the same, heance the return FALSE
+			 */
+			g_tree_insert(t, (gpointer) entry_dup(&e), VALUE);
+			g_free(e.f_target);
+			return FALSE;
+		}
 
 		g_tree_insert(t, (gpointer) entry_dup(&e), VALUE);
 		*c = '/';
@@ -70,15 +87,16 @@ dir_prepend(GTree *t, char *path)
 }
 
 void
-dir_crawl(GTree *t, GHashTable *linkhash, char *path)
+dir_crawl(GTree *t, GHashTable *linkhash, GHashTable *userhash,
+		GHashTable *grouphash, char *path)
 {
 	DIR 		*dir;
 	struct dirent 	*dent;
-	struct r_entry  *directory;
+	struct rdup     *directory;
 	char 		*curpath;
 	gchar		*lnk;
 	struct stat   	s;
-	struct r_entry  pop;
+	struct rdup     pop;
 	struct remove_path rp;
 	dev_t 		current_dev;
 	size_t 		curpath_len;
@@ -86,8 +104,8 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 	/* dir stack */
 	gint32 d = 0;
 	gint32 dstack_cnt  = 1;
-	struct r_entry **dirstack =
-		g_malloc(dstack_cnt * D_STACKSIZE * sizeof(struct r_entry *));
+	struct rdup **dirstack =
+		g_malloc(dstack_cnt * D_STACKSIZE * sizeof(struct rdup *));
 
 	if (!(dir = opendir(path))) {
 		/* non-dirs are also allowed, check for this, if it isn't give the error */
@@ -106,7 +124,7 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 #else
 	if (fstat(rdup_dirfd(dir), &s) != 0) {
 #endif
-		msg(_("Cannot determine holding device of the directory `%s\': %s"), path, 
+		msg(_("Cannot determine holding device of the directory `%s\': %s"), path,
 				strerror(errno));
 		closedir(dir);
 		g_free(dirstack);
@@ -115,7 +133,7 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 	current_dev = s.st_dev;
 
 	while((dent = readdir(dir))) {
-		if (!strcmp(dent->d_name, ".") || 
+		if (!strcmp(dent->d_name, ".") ||
 				!strcmp(dent->d_name, ".."))
 			continue;
 
@@ -139,15 +157,20 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 			continue;
 		}
 
-		if (S_ISREG(s.st_mode) || S_ISLNK(s.st_mode) || 
+		if (S_ISREG(s.st_mode) || S_ISLNK(s.st_mode) ||
 				S_ISBLK(s.st_mode) || S_ISCHR(s.st_mode) ||
 				S_ISFIFO(s.st_mode) || S_ISSOCK(s.st_mode) ) {
 
 			pop.f_name      = curpath;
+			pop.f_target	= NULL;
 			pop.f_name_size = curpath_len;
 			pop.f_uid       = s.st_uid;
+			pop.f_user      = lookup_user(userhash, pop.f_uid);
 			pop.f_gid       = s.st_gid;
+			pop.f_group     = lookup_group(grouphash, pop.f_gid);
 			pop.f_ctime     = s.st_ctime;
+			pop.f_mtime     = s.st_mtime;
+			pop.f_atime     = s.st_atime;
 			pop.f_mode      = s.st_mode;
 			pop.f_size      = s.st_size;
 			pop.f_dev       = s.st_dev;
@@ -179,21 +202,32 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 
 			/* hardlinks */
 			if (s.st_nlink > 1) {
-				if ((lnk = hard_link(linkhash, &pop))) {
-					/* we got a match back, modify pop */
-					pop = *(sym_link(&pop, lnk));
+				if ((lnk = hlink(linkhash, &pop))) {
+					pop.f_target = lnk;
+					pop.f_lnk  = 1;
 				}
 			}
-			if (S_ISLNK(s.st_mode)) 
-				pop = *(sym_link(&pop, NULL));
+			if (S_ISLNK(s.st_mode))
+				pop.f_target = slink(&pop);
+
+
+			if (S_ISLNK(s.st_mode) || pop.f_lnk) {
+				/* fix the name and the sizes */
+				pop.f_size = pop.f_name_size;
+				pop.f_name_size += 4 + strlen(pop.f_target);
+			}
+
 
 			g_tree_insert(t, (gpointer) entry_dup(&pop), VALUE);
+
+			if (S_ISLNK(s.st_mode))
+				g_free(pop.f_target);
 			g_free(curpath);
 			continue;
 		} else if(S_ISDIR(s.st_mode)) {
 			/* one filesystem */
 			if (opt_onefilesystem && s.st_dev != current_dev) {
-				msg(_("Walking into different filesystem"));
+				msg(_("Not walking into different filesystem `%s\'"), curpath);
 				g_free(curpath);
 				continue;
 			}
@@ -202,13 +236,17 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 				g_free(curpath);
 				continue;
 			}
-
-			dirstack[d] = g_malloc(sizeof(struct r_entry));
-			dirstack[d]->f_name       = g_strdup(curpath); 
+			dirstack[d] = g_malloc(sizeof(struct rdup));
+			dirstack[d]->f_name       = g_strdup(curpath);
+			dirstack[d]->f_target	  = NULL;
 			dirstack[d]->f_name_size  = curpath_len;
 			dirstack[d]->f_uid	  = s.st_uid;
+			dirstack[d]->f_user	  = lookup_user(userhash, s.st_uid);
 			dirstack[d]->f_gid        = s.st_gid;
+			dirstack[d]->f_group      = lookup_group(grouphash, s.st_gid);
 			dirstack[d]->f_ctime      = s.st_ctime;
+			dirstack[d]->f_mtime      = s.st_mtime;
+			dirstack[d]->f_atime      = s.st_atime;
 			dirstack[d]->f_mode       = s.st_mode;
 			dirstack[d]->f_size       = s.st_size;
 			dirstack[d]->f_dev        = s.st_dev;
@@ -217,9 +255,9 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 			dirstack[d]->f_lnk        = 0;
 
 			if (d++ % D_STACKSIZE == 0) {
-				dirstack = g_realloc(dirstack, 
-						++dstack_cnt * D_STACKSIZE * 
-						sizeof(struct r_entry *));
+				dirstack = g_realloc(dirstack,
+						++dstack_cnt * D_STACKSIZE *
+						sizeof(struct rdup *));
 			}
 			g_free(curpath);
 			continue;
@@ -231,14 +269,24 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 		}
 	}
 	closedir(dir);
+	if (opt_atime) {
+		/* reset dirs atime */
+		if (d > 0 && opt_atime) {
+			struct utimbuf ut;
+			ut.actime  = dirstack[d - 1]->f_atime;
+			ut.modtime = dirstack[d - 1]->f_mtime;
+			if (utime(dirstack[d - 1]->f_name, &ut) == -1)
+				msg(_("Failed to reset atime: '%s\': %s"), dirstack[d - 1]->f_name, strerror(errno));
+		}
+	}
 
 	while (d > 0) {
-		directory = dirstack[--d]; 
+		directory = dirstack[--d];
 		g_tree_insert(t, (gpointer) entry_dup(directory), VALUE);
 		/* recurse */
 		/* potentially expensive operation. Better would be to when we hit
 		 * .nobackup to go up the tree and delete some nodes.... or not */
-		dir_crawl(t, linkhash, directory->f_name);
+		dir_crawl(t, linkhash, userhash, grouphash, directory->f_name);
 		entry_free(directory);
 	}
 	g_free(dirstack);
