@@ -1,5 +1,5 @@
-/* 
- * Copyright (c) 2009 Miek Gieben
+/*
+ * Copyright (c) 2009 - 2011 Miek Gieben
  * See LICENSE for the license
  * make a dir writable and later remove that
  * right again
@@ -15,12 +15,17 @@ dir_write(gchar *p)
 		return NULL;
 
 	struct stat *s = g_malloc(sizeof(struct stat));
+#ifdef DEBUG
+	msgd(__func__, __LINE__,_("Making directory writable `%s\'"), p);
+#endif /* DEBUG */
 
 	if (stat(p, s) == -1)
 		return NULL;
 
 	/* make it writable, assume we are the OWNER */
-	chmod(p, s->st_mode | S_IWUSR);
+	if (chmod(p, s->st_mode | S_IWUSR) == -1) {
+		msgd(__func__, __LINE__,_("Failed to make directory writeable `%s\': %s"), p, strerror(errno));
+	}
 	return s;
 }
 
@@ -30,7 +35,9 @@ dir_restore(gchar *p, struct stat *s)
 	if (!s || !p)
 		return;
 	/* restore perms - assumes *s has not be f*cked up */
-	chmod(p, s->st_mode & ~S_IFMT);
+	if (chmod(p, s->st_mode & 07777) == -1) {
+		msgd(__func__, __LINE__,_("Failed to restore permissions `%s\': %s"), p, strerror(errno));
+	}
 }
 
 /**
@@ -42,16 +49,20 @@ dir_parent(gchar *p)
 {
 	gchar *p2;
 	gchar *n;
+	gchar *copy;
 
 	if (!p)
 		return NULL;
+
 	if (p[0] == '/' && p[1] == '\0')
 		return p;
 
-	n = strrchr(p, '/'); 
+	copy = g_strdup(p);
+	n = strrchr(copy, '/');
 	if (n) {
 		*(n+1) = '\0';
-		p2 = g_strdup(p);
+		p2 = g_strdup(copy);
+		g_free(copy);
 		*n= '/';
 		return p2;
 	}
