@@ -1,7 +1,7 @@
-/* 
- * Copyright (c) 2009 Miek Gieben
+/*
+ * Copyright (c) 2009 - 2011 Miek Gieben
  * See LICENSE for the license
- * rdup-up -- update an directory tree with 
+ * rdup-up -- update an directory tree with
  * and rdup archive
  */
 
@@ -13,46 +13,51 @@ char *PROGNAME="rdup-up";
 gint opt_verbose 	   = 0;                         /* be more verbose */
 gint opt_output		   = O_RDUP;			/* set these 2 so we can use parse_entry */
 gint opt_input	           = I_RDUP;
+gboolean opt_quiet         = FALSE;                     /* don't complain about chown() errors */
 gboolean opt_dry	   = FALSE;			/* don't touch the filesystem */
+gboolean opt_table	   = FALSE;			/* table of contents */
 gboolean opt_top	   = FALSE;			/* create top dir if it does not exist */
 gchar *opt_path_strip	   = NULL;			/* -r PATH, strip PATH from pathnames */
 guint opt_path_strip_len   = 0;				/* number of path labels in opt_path_strip */
 guint opt_strip		   = 0;				/* -s: strippath */
-sig_atomic_t sig           = 0;
-GSList *hlink		   = NULL;			/* save hardlink for post processing */		
+int sig           = 0;
+GSList *hlink_list		   = NULL;			/* save hardlink for post processing */		
 extern int opterr;
 int opterr		   = 0;
 
-
 /* update the directory with the archive */
 static gboolean
 update(char *path)
 {
-	struct r_entry *rdup_entry;
+	struct rdup    *rdup_entry;
 	size_t         line, i, pathsize;
 	size_t	       pathlen;
 	char           *buf, *pathbuf, *n, *p;
 	char           delim;
-	FILE           *fp;
-	struct stat    s;
 	gboolean       ok;
+	GHashTable     *uidhash; /* holds username -> uid */
+	GHashTable     *gidhash; /* holds groupname -> gid */
 
 	buf	= g_malloc(BUFSIZE + 1);
 	pathbuf = g_malloc(BUFSIZE + 1);
 	i       = BUFSIZE;
-	fp  	= stdin;
 	delim   = '\n';
 	line    = 0;
 	ok      = TRUE;
-	pathlen = strlen(path);
+	uidhash = g_hash_table_new(g_str_hash, g_str_equal);
+	gidhash = g_hash_table_new(g_str_hash, g_str_equal);
+	if (path)
+		pathlen = strlen(path);
+	else
+		pathlen = 0;
 
-	while ((rdup_getdelim(&buf, &i, delim, fp)) != -1) {
+	while ((rdup_getdelim(&buf, &i, delim, stdin)) != -1) {
 		line++;
 		n = strrchr(buf, '\n');
-		if (n) 
+		if (n)
 			*n = '\0';
 
-		if (!(rdup_entry = parse_entry(buf, line, &s, NO_STAT_CONTENT))) {
+		if (!(rdup_entry = parse_entry(buf, line))) {
 			/* msgs from entry.c */
 			exit(EXIT_FAILURE);
 		}
@@ -71,19 +76,30 @@ update(char *path)
 			exit(EXIT_FAILURE);
 		}
 
-		/* strippath must be inserted here */
 		rdup_entry->f_name = pathbuf;
+
+		/* extract target from rdup_entry */
+		if (S_ISLNK(rdup_entry->f_mode) || rdup_entry->f_lnk) {
+			// filesize is spot where to cut
+			rdup_entry->f_name[rdup_entry->f_size] = '\0';
+			rdup_entry->f_target = rdup_entry->f_name +
+				rdup_entry->f_size + 4;
+		} else {
+			rdup_entry->f_target = NULL;
+		}
+
+		/* strippath must be inserted here */
 		if (opt_strip)
 			strippath(rdup_entry);
 
-		if (opt_path_strip) 
+		if (opt_path_strip)
 			strippathname(rdup_entry);
 
 		if (!rdup_entry->f_name)
 			p = NULL;
 		else {
 			/* avoid // at the beginning */
-			if (pathlen == 1 && path[0] == '/') {
+			if ( (pathlen == 1 && path[0] == '/') || pathlen == 0 ) {
 				p = g_strdup(rdup_entry->f_name);
 			} else {
 				p = g_strdup_printf("%s%s", path, rdup_entry->f_name);
@@ -93,12 +109,17 @@ update(char *path)
 			}
 		}
 		rdup_entry->f_name = p;
-		if (mk_obj(stdin, path, rdup_entry) == FALSE)
+		if (mk_obj(stdin, path, rdup_entry, uidhash, gidhash) == FALSE)
 			ok = FALSE;
+//	Look into this XXX
+//		g_free(rdup_entry->f_name);
+//		g_free(rdup_entry->f_user);
+//		g_free(rdup_entry->f_group);
+//		g_free(rdup_entry);
 	}
 
 	/* post-process hardlinks */
-	if (mk_hlink(hlink) == FALSE)
+	if (mk_hlink(hlink_list) == FALSE)
 		ok = FALSE;
 
 	g_free(buf);
@@ -147,10 +168,13 @@ main(int argc, char **argv)
 		}
 	}
 
-	while ((c = getopt (argc, argv, "thnVvs:r:")) != -1) {
+	while ((c = getopt (argc, argv, "thnVvs:r:Tq")) != -1) {
 		switch (c) {
 			case 'v':
-				opt_verbose++;
+				if (!opt_verbose)
+					opt_verbose++;
+				else
+					opt_verbose = 0;
 				break;
 			case 'h':
 				usage_up(stdout);
@@ -158,23 +182,15 @@ main(int argc, char **argv)
 			case 'n':
 				opt_dry = TRUE;
 				break;
+			case 'T':
+				opt_table = TRUE;
+				opt_dry = TRUE;
+				opt_verbose = 0;
+				break;
 			case 's':
-				if (opt_path_strip != NULL) {
-					msg(_("The -r and -s option can not be used together"));
-					exit(EXIT_FAILURE);
-				}
                                 opt_strip = abs(atoi(optarg));
                                 break;
 			case 'r':
-				if (opt_strip != 0) {
-					msg(_("The -r and -s option can not be used together"));
-					exit(EXIT_FAILURE);
-				}
-				if (strlen(optarg) == 0) {		/* does this help? XX */
-					msg(_("-r needs an arugment"));
-					exit(EXIT_FAILURE);
-				}
-
 				/* expand relative paths */
 				if (!g_path_is_absolute(optarg))
 					opt_path_strip = abspath(g_strdup_printf("%s/%s", pwd, optarg));
@@ -197,11 +213,18 @@ main(int argc, char **argv)
 				}
 				opt_path_strip_len--;	/* we added the closing slash, so need -1 here */
 				break;
+                        case 'q':
+                                opt_quiet = TRUE;
+                                break;
 			case 't':
 				opt_top = TRUE;
 				break;
 			case 'V':
-				fprintf(stdout, "%s %s\n", PROGNAME, VERSION);
+#ifdef DEBUG
+                                fprintf(stdout, "%s %s (with --enable-debug)\n", PROGNAME, VERSION);
+#else
+                                fprintf(stdout, "%s %s\n", PROGNAME, VERSION);
+#endif /* DEBUG */
 				exit(EXIT_SUCCESS);
 			default:
 				msg(_("Unknown option seen `%c\'"), optopt);
@@ -211,14 +234,21 @@ main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	if (argc != 1) {
-		msg(_("Destination directory is required"));
-		exit(EXIT_FAILURE);
+	if (opt_table) {
+		path = NULL;
+	} else {
+		if (argc != 1) {
+			msg(_("Destination directory is required"));
+			exit(EXIT_FAILURE);
+		}
+		if (!g_path_is_absolute(argv[0])) {
+			gchar* full_path = g_strdup_printf("%s/%s", pwd, argv[0]);
+			path = abspath(full_path);
+			g_free(full_path);
+		}
+		else
+			path = abspath(argv[0]);
 	}
-	if (!g_path_is_absolute(argv[0])) 
-		path = abspath(g_strdup_printf("%s/%s", pwd, argv[0]));
-	else
-		path = abspath(argv[0]);
 
 	if (!opt_dry) {
 		if (opt_top) {
@@ -242,5 +272,7 @@ main(int argc, char **argv)
 	if (update(path) == FALSE)
 		exit(EXIT_FAILURE);
 
+	g_free(path);
+
 	exit(EXIT_SUCCESS);
 }
