@@ -1,5 +1,5 @@
-/* 
- * Copyright (c) 2005 - 2009 Miek Gieben
+/*
+ * Copyright (c) 2005 - 2011 Miek Gieben
  * See LICENSE for the license
  */
 
@@ -7,21 +7,25 @@
 
 char *PROGNAME="rdup";
 /* options */
-gboolean opt_null 	   = FALSE;                   /* delimit all in/output with \0  */
 gboolean opt_onefilesystem = FALSE;   		      /* stay on one filesystem */
 gboolean opt_nobackup      = TRUE;             	      /* don't ignore .nobackup files */
 gboolean opt_removed       = TRUE; 		      /* whether to print removed files */
 gboolean opt_modified      = TRUE; 		      /* whether to print modified files */
 gboolean opt_reverse	   = FALSE;		      /* whether to reverse print the lists */
-char *opt_format 	   = "%p%T %b %u %g %l %s %n\n"; /* format of rdup output */
+gboolean opt_tty	   = FALSE;		      /* force write to tty */
+gboolean opt_atime	   = FALSE;			      /* reset access time */
+char *opt_format 	   = "%p%T %b %t %u %U %g %G %l %s\n%n%C"; /* format of rdup output */
+#if 0
+char *opt_format 	   = "%p%T %b %t %u %U %g %G %l %s %n\n";
+#endif
 gint opt_verbose 	   = 0;                       /* be more verbose */
-gboolean opt_atime	   = 0;			      /* reset access time */
 size_t opt_size            = 0;                       /* only output files smaller then <size> */
 time_t opt_timestamp       = 0;                       /* timestamp file c|m time */
 
-sig_atomic_t sig           = 0;
+int sig		           = 0;
 extern int opterr;
 int opterr		   = 0;
+GSList *child		   = NULL;
 
 #define CORRUPT(x)	{ \
 			msg((x), l); \
@@ -42,7 +46,7 @@ g_tree_subtract(GTree *a, GTree *b)
 	diff = g_tree_new(gfunc_equal);
 	s.d = diff;
 	s.b = b;
-	/* everything in a, but NOT in b 
+	/* everything in a, but NOT in b
 	 * diff gets filled inside this function */
 	g_tree_foreach(a, gfunc_subtract, (gpointer)&s);
 	return diff;
@@ -55,15 +59,11 @@ g_tree_subtract(GTree *a, GTree *b)
 static GTree *
 g_tree_read_file(FILE *fp)
 {
-	char 	      *buf;
-	char          *n;
-	char          *p;
-	char 	      *q;
-	char 	      delim;
-	char	      linktype;
+	gchar 	      *buf, *n, *p, *q;
+	gchar 	      delim, linktype;
 	mode_t        modus;
 	GTree         *tree;
-	struct r_entry *e;
+	struct rdup   *e;
 	size_t        s;
 	size_t 	      l;
 	size_t        f_name_size;
@@ -73,17 +73,13 @@ g_tree_read_file(FILE *fp)
 	ino_t	      f_ino;
 
 	tree = g_tree_new(gfunc_equal);
-	buf  = g_malloc(BUFSIZE + 1);
-	s    = BUFSIZE;
-	l    = 1;
 
 	if (!fp)
 	    return tree;
-
-	if (opt_null)
-		delim = '\0';
-	else
-		delim = '\n';
+	buf  = g_malloc(BUFSIZE + 1);
+	s    = BUFSIZE;
+	l    = 1;
+	delim= '\n';
 
 	while ((rdup_getdelim(&buf, &s, delim, fp)) != -1) {
 		if (sig != 0) {
@@ -92,106 +88,123 @@ g_tree_read_file(FILE *fp)
 		}
 
 		/* comment */
-		if (buf[0] == '#') 
+		if (buf[0] == '#')
 			continue;
 
-		if (s < LIST_MINSIZE) 
-			CORRUPT("Corrupt entry in filelist at line: %zd"); 
+		if (s < LIST_MINSIZE)
+			CORRUPT("Corrupt entry at line: %zd, line to short");
 
-		if (!opt_null) {
-			n = strrchr(buf, '\n');
-			if (n)
-				*n = '\0';
-		}
+		n = strrchr(buf, '\n');
 
 		/* get modus */
 		if (buf[LIST_SPACEPOS] != ' ')
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
+			CORRUPT("Corrupt entry at line: %zd, no space found");
 
 		buf[LIST_SPACEPOS] = '\0';
 		modus = (mode_t)atoi(buf);
 		if (modus == 0)
-			CORRUPT("Corrupt entry in filelist at line: %zd, mode should be numerical");
+			CORRUPT("Corrupt entry at line: %zd, mode should be numerical");
 
 		/* the dev */
 		q = buf + LIST_SPACEPOS + 1;
 		p = strchr(buf + LIST_SPACEPOS + 1, ' ');
 		if (!p)
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
+			CORRUPT("Corrupt entry at line: %zd, no space found");
 		
 		*p = '\0';
 		f_dev = (dev_t)atoi(q);
 		if (f_dev == 0)
-			CORRUPT("Corrupt entry in filelist at line: %zd, zero device");
+			CORRUPT("Corrupt entry at line: %zd, zero device");
 
 		/* the inode */
 		q = p + 1;
 		p = strchr(p + 1, ' ');
-		if (!p) 
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
+		if (!p)
+			CORRUPT("Corrupt entry at line: %zd, no space found");
 		
 		*p = '\0';
 		f_ino = (ino_t)atoi(q);
 		if (f_ino == 0)
-			CORRUPT("Corrupt entry in filelist at line: %zd, zero inode");
+			CORRUPT("Corrupt entry at line: %zd, zero inode");
 
-		/* hardlink or anything else: h or * */
+		/* hardlink/link or anything else: h/l or * */
 		q = p + 1;
 		p = strchr(p + 1, ' ');
-		if (!p) 
-			CORRUPT("Corrupt entry in filelist at line: %zd, no link information found");
+		if (!p)
+			CORRUPT("Corrupt entry at line: %zd, no link information found");
 
 		linktype = *q;
-		if (linktype != '*' && linktype != 'h')
-			CORRUPT("Illegal link type as line: %zd");
+		if (linktype != '-' && linktype != 'h' && linktype != 'l')
+			CORRUPT("Illegal link type at line: %zd");
 
 		/* skip these for now - but useful to have */
 		/* uid */
 		q = p + 1;
 		p = strchr(p + 1, ' ');
 		if (!p)
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
+			CORRUPT("Corrupt entry at line: %zd, no space found");
 		
 		/* gid */
 		q = p + 1;
 		p = strchr(p + 1, ' ');
 		if (!p)
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
+			CORRUPT("Corrupt entry at line: %zd, no space found");
 
 		/* the path size */
 		q = p + 1;
 		p = strchr(p + 1, ' ');
 		if (!p) {
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
+			CORRUPT("Corrupt entry at line: %zd, no space found");
 		}
 		*p = '\0';
 		f_name_size = (size_t)atoi(q);
+		if (f_name_size == 0)
+			CORRUPT("Pathname lenght can not be zero at line: %zd");
 
 		/* filesize */
 		q = p + 1;
 		p = strchr(p + 1, ' ');
 		if (!p)
-			CORRUPT("Corrupt entry in filelist at line: %zd, no space found");
-		
+			CORRUPT("Corrupt entry at line: %zd, no space found");
+
 		*p = '\0';
 		f_size = (size_t)atoi(q);
 
+		/* with getdelim we read the delimeter too kill it here */
 		str_len = strlen(p + 1);
+		if (str_len == 1)
+			CORRUPT("Actual pathname length can not be zero at line: %zd");
+
+		p[str_len] = '\0'; str_len--;
 		if (str_len != f_name_size) {
-			msg(_("Corrupt entry in filelist at line: %zd, length `%zd\' does not match `%zd\'"), l,
+			msg(_("Corrupt entry at line: %zd, length `%zd\' does not match `%zd\'"), l,
 					str_len, f_name_size);
-			l++; 
+			l++;
 			continue;
 		}
 
-		e = g_malloc(sizeof(struct r_entry));
+		e = g_malloc(sizeof(struct rdup));
 		e->f_name      = g_strdup(p + 1);
-		e->f_name_size = f_name_size;
-		e->f_size      = f_size;
+
+		if (linktype == 'h' || linktype == 'l') {
+			e->f_name_size    = strlen(e->f_name);
+			e->f_name[f_size] = '\0'; /* set NULL just before the ' -> ' */
+			e->f_size         = strlen(e->f_name);
+			e->f_target       = e->f_name + f_size + 4;
+		} else {
+			e->f_name_size = f_name_size;
+			e->f_target    = NULL;
+			e->f_size      = f_size;
+		}
+
 		e->f_mode      = modus;
 		e->f_uid       = 0;	/* keep this 0 for now */
 		e->f_gid       = 0;	/* keep this 0 for now */
 		e->f_ctime     = 0;
+		e->f_mtime     = 0;
+		e->f_atime     = 0;
+		e->f_user      = NULL;
+		e->f_group     = NULL;
 		e->f_dev       = f_dev;
 		e->f_ino       = f_ino;
 		if (linktype == 'h')
@@ -230,15 +243,21 @@ main(int argc, char **argv)
 	GTree	*new;		/* all that is new */
 	GTree	*changed;	/* all that is possibly changed */
 	GHashTable *linkhash;	/* hold dev, inode, name stuff */
+	GHashTable *userhash;	/* holds uid -> username */
+	GHashTable *grouphash;  /* holds gid -> groupname */
+	struct utimbuf ut;	/* time to set on timestamp file */
+
 	FILE 	*fplist;
 	gint    i;
 	int 	c;
 	char    pwd[BUFSIZE + 1];
-	char	*path;
-	char    *time;
+	char	*path, *stamp, *q, *r;
+	gchar   **args;
 	gboolean devnull = FALSE;	/* hack: remember if we open /dev/null */
-
 	struct sigaction sa;
+
+	ut.actime = time(NULL);
+	ut.modtime = ut.actime;
 	
 	/* i18n, set domain to rdup */
 #ifdef ENABLE_NLS
@@ -259,10 +278,12 @@ main(int argc, char **argv)
 
 	curtree = g_tree_new(gfunc_equal);
 	backup  = g_tree_new(gfunc_equal);
-	linkhash = g_hash_table_new(g_str_hash, g_str_equal);
+	linkhash  = g_hash_table_new(g_str_hash, g_str_equal);
+	grouphash = g_hash_table_new(g_int_hash, g_int_equal);
+	userhash  = g_hash_table_new(g_int_hash, g_int_equal);
 	remove  = NULL;
 	opterr = 0;
-	time = NULL;
+	stamp = NULL;
 
 	if (((getuid() != geteuid()) || (getgid() != getegid()))) {
 		msg(_("Will not run suid/sgid for safety reasons"));
@@ -280,44 +301,84 @@ main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
-
-	while ((c = getopt (argc, argv, "acrlmhVRnd:N:M:s:vqx0F:E:")) != -1) {
+#ifdef DEBUG
+	msgd(__func__, __LINE__, _("DEBUG is enabled!"));
+#endif /* DEBUG */
+	while ((c = getopt (argc, argv, "acrlmhVRnd:N:M:P:s:vqxF:E:")) != -1) {
 		switch (c) {
 			case 'F':
 				opt_format = optarg;
 				break;
 			case 'E':
-				if (!regexp_init(optarg)) 
+				if (!regexp_init(optarg))
 					exit(EXIT_FAILURE);
 				break;
 			case 'a':
 				opt_atime = TRUE;
+				/* when atime is true, every file is touched during the
+				 * backup (the c_time changes). To make rdup not see these
+				 * files as new in the backup, we must set the timestamp
+				 * file with a timestamp AFTER the backup.
+				 * If we do this we will not see file the are changed
+				 * DURING the backup...
+				 */
 				break;
 			case 'c':
-				opt_format = "%p%T %b %u %g %l %s\n%n%C";
+				opt_tty = TRUE;
 				break;
 			case 'h':
 				usage(stdout);
 				exit(EXIT_SUCCESS);
 			case 'V':
+#ifdef DEBUG
+				fprintf(stdout, "%s %s (with --enable-debug)\n", PROGNAME, VERSION);
+#else
 				fprintf(stdout, "%s %s\n", PROGNAME, VERSION);
+#endif /* DEBUG */
+
 				exit(EXIT_SUCCESS);
 			case 'n':
 				opt_nobackup = FALSE;
 				break;
-			case 'N': 
+			case 'N':
 				opt_timestamp = timestamp(optarg, TRUE);
-				time = optarg;
+				stamp = optarg;
 				break;
 			case 'M':
 				opt_timestamp = timestamp(optarg, FALSE);
-				time = optarg;
+				stamp = optarg;
 				break;
 			case 'R':
 				opt_reverse = TRUE;
 				break;
+			case 'P':
+                                /* allocate new for each child */
+                                args = g_malloc((MAX_CHILD_OPT + 2) * sizeof(char *));
+                                q = g_strdup(optarg);
+                                /* this should be a comma seprated list
+                                 * arg0,arg1,arg2,...,argN */
+                                r = strchr(q, ',');
+                                if (!r) {
+                                        args[0] = q;
+                                        args[1] = NULL;
+                                } else {
+                                        *r = '\0';
+                                        for(i = 0; r; r = strchr(r + 1, ','), i++) {
+                                                if (i > MAX_CHILD_OPT) {
+                                                        msg(_("Only %d extra args per child allowed"), MAX_CHILD_OPT);
+                                                        exit(EXIT_FAILURE);
+                                                }
+                                                *r = '\0';
+                                                args[i] = g_strdup(q);
+                                                q = r + 1;
+                                        }
+                                        args[i] = g_strdup(q);
+                                        args[i + 1] = NULL;
+                                }
+                                child = g_slist_append(child, args);
+                                break;
 			case 'v':
-				opt_verbose++; 
+				opt_verbose++;
 				if (opt_verbose > 2) {
 					opt_verbose = 2;
 				}
@@ -333,9 +394,6 @@ main(int argc, char **argv)
 			case 'x':
 				opt_onefilesystem = TRUE;
 				break;
-			case '0':
-				opt_null = TRUE;
-				break;
 			case 's':
 				opt_size = atoi(optarg);
 				if (opt_size == 0) {
@@ -356,8 +414,15 @@ main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	/* be as irritating as rdup-tr */
+	if (!opt_tty && isatty(1) == 1) {
+		msg(_("Will not write to a tty"));
+		exit(EXIT_FAILURE);
+	}
+
 	if (argc == 1) {
 		/* default to . as the dir to dump */
+		msg(_("No directory given, dumping `.\'"));
 		argv[1] = g_strdup(".");
 		argc++;
 	}
@@ -371,10 +436,14 @@ main(int argc, char **argv)
 		curtree = g_tree_read_file(fplist);
 		fclose(fplist);
 	}
-	
+
 	for (i = 1; i < argc; i++) {
-		if (!g_path_is_absolute(argv[i])) 
-		    path = abspath(g_strdup_printf("%s/%s", pwd, argv[i]));
+		if (!g_path_is_absolute(argv[i]))
+		{
+		    char* path_tmp = g_strdup_printf("%s/%s", pwd, argv[i]);
+		    path = abspath(path_tmp);
+		    g_free(path_tmp);
+		}
 		else
 		    path = abspath(argv[i]);
 
@@ -384,12 +453,11 @@ main(int argc, char **argv)
 		}
 
 		/* add dirs leading up the dir/file */
-		if (!dir_prepend(backup, path)) {
-			msg(_("Skipping `%s\'"), path);
-			continue;
-		}
+		if (!dir_prepend(backup, path, userhash, grouphash)) continue;
+
 		/* descend into the dark, misty directory */
-		dir_crawl(backup, linkhash, path);
+		dir_crawl(backup, linkhash, userhash, grouphash, path);
+		g_free(path);
 	}
 
 	/* everything that is gone from the filesystem */
@@ -398,55 +466,59 @@ main(int argc, char **argv)
 	/* everything that is really new on the filesystem */
 	new     = g_tree_subtract(backup, curtree);
 
-	/* all stuff that should be ctime checked, to see if it has 
-	 * changed */
+	/* all stuff that should be ctime checked, to see if it has changed */
 	changed = g_tree_subtract(backup, new);
 	/* some dirs might still linger in changed, while they are in fact
 	 * removed, kill those here */
 	changed = g_tree_subtract(changed, remove);
 
-	/* first what to remove, then what to backup */
+#ifdef DEBUG
+	/* we first crawled the disk to see what is changed
+	 * then we output. If we wait here a few seconds
+	 * we can remove files that should have been
+	 * added.  */
+	msg(_("DEBUG: sleeping for a while"));
+	sleep(5);
+#endif /* DEBUG */
 
+	/* first what to remove, then what to backup */
 	if (opt_reverse) {
 		GList *list_remove, *list_changed, *list_new = NULL;
 		list_remove = reverse(remove);
 		list_changed = reverse(changed);
 		list_new = reverse(new);
-		g_list_foreach(list_remove, gfunc_remove_list, NULL); 
-		g_list_foreach(list_changed, gfunc_backup_list, NULL); 
-		g_list_foreach(list_new, gfunc_new_list, NULL); 
+		g_list_foreach(list_remove, gfunc_remove_list, NULL);
+		g_list_foreach(list_changed, gfunc_backup_list, NULL);
+		g_list_foreach(list_new, gfunc_new_list, NULL);
 	} else {
 		g_tree_foreach(remove, gfunc_remove, NULL);
 		g_tree_foreach(changed, gfunc_backup, NULL);
 		g_tree_foreach(new, gfunc_new, NULL);
 	}
 
-	/* write new filelist */
+	/* write new list */
 	if (!devnull) {
 	    if (!(fplist = fopen(argv[0], "w"))) {
 		    msg(_("Could not write filelist `%s\': %s"), argv[0], strerror(errno));
 	    } else {
 		/* write temporary file, add little comment */
-		fprintf(fplist, 
+		fprintf(fplist,
 			"# mode dev inode linktype uid gid pathlen filesize path\n");
 		g_tree_foreach(backup, gfunc_write, fplist);
 		fclose(fplist);
 	    }
 	}
-
 	/* re-touch the timestamp file */
-	if (time && (creat(time, S_IRUSR | S_IWUSR) == -1)) {
-		msg(_("Could not create timestamp file `%s\': %s"), time, strerror(errno));
-		exit(EXIT_FAILURE);
+	if (stamp) {
+		if (creat(stamp, S_IRUSR | S_IWUSR) == -1) {
+			msg(_("Could not create timestamp file `%s\': %s"), stamp, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+		/* and set the time when rdup was started, only when -a was not given */
+		if (! opt_atime && utime(stamp, &ut) == -1) {
+			msg(_("Failed to reset atime: '%s\': %s"), stamp, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
 	}
-
-/*	
-	g_tree_foreach(curtree, gfunc_free, NULL);
-	g_tree_foreach(backup, gfunc_free, NULL);
-*/
-	g_tree_destroy(curtree);
-	g_tree_destroy(backup);
-	g_tree_destroy(remove);
-	g_hash_table_destroy(linkhash);
 	exit(EXIT_SUCCESS);
 }
