@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 - 2008 Miek Gieben
+ * Copyright (c) 2005 - 2011 Miek Gieben
  * See LICENSE for the license
  *
  * regexp helper functions
@@ -8,7 +8,6 @@
 #include "rdup.h"
 #include <pcre.h>
 
-extern gboolean opt_null;
 GSList *pregex_list = NULL;
 
 /**
@@ -16,7 +15,8 @@ GSList *pregex_list = NULL;
  * in a linked list
  */
 gboolean
-regexp_init(char *file) {
+regexp_init(char *file)
+{
 	FILE 	      	*fp;
 	char 		*buf;
 	const char	*errbuf;
@@ -36,15 +36,11 @@ regexp_init(char *file) {
 	/* read the regexp */
 	buf  = g_malloc(BUFSIZE + 1);
 	s    = BUFSIZE;
-
-	if (opt_null)
-                delim = '\0';
-        else 
-                delim = '\n';
+        delim = '\n';
 
 	l = 1;
 	while ((j = rdup_getdelim(&buf, &s, delim, fp)) != -1) {
-		if (buf[0] == '#' || buf[0] == '\n') 
+		if (buf[0] == '#' || buf[0] == '\n')
 			continue;
 
 		/* buf[j - 1] holds the delimeter */
@@ -53,7 +49,7 @@ regexp_init(char *file) {
 		if ((P = pcre_compile(buf, 0, &errbuf, &erroff, NULL)) == NULL) {
 			/* error */
 			fclose(fp);
-			msg(_("Corrupt regular expression line: %zd, column %d: %s"), l, erroff, errbuf); 
+			msg(_("Corrupt regular expression line: %zd, column %d: %s"), l, erroff, errbuf);
 			g_free(buf);
 			return FALSE;
 		} else {
