@@ -3,14 +3,24 @@
    */
 
 #include "rdup.h"
-#include <nettle/sha.h>
 
-int
-sha1_stream(__attribute__((unused)) FILE *f, unsigned char *digest)
-{
 #ifdef HAVE_LIBNETTLE
-	uint8_t buffer[SHA1_DIGEST_SIZE + 1];
+#include <nettle/sha.h>
+#endif /* HAVE_LIBNETTLE */
+
+#ifndef HAVE_LIBNETTLE
+int
+sha1_stream( __attribute__((unused)) FILE *f, __attribute__((unused)) unsigned char *digest)
+#else
+int
+sha1_stream(FILE *f, unsigned char *digest)
+#endif
+
+{
+
+#ifdef HAVE_LIBNETTLE
 	struct sha1_ctx ctx;
+	uint8_t buffer[SHA1_DIGEST_SIZE];
 
 	sha1_init(&ctx);
 	for (;;)
@@ -24,11 +34,6 @@ sha1_stream(__attribute__((unused)) FILE *f, unsigned char *digest)
 		return -1;
 
 	sha1_digest(&ctx, SHA1_DIGEST_SIZE, digest);
-#else
-	int i;
-	for(i = 0; i < SHA1_DIGEST_SIZE; i++) {
-		digest[i] = '\0';
-	}
 #endif /* HAVE_LIBNETTLE */
-	return 0;  
+	return 0;
 }
