@@ -1,11 +1,11 @@
-/* 
- * Copyright (c) 2005 - 2008 Miek Gieben
+/*
+ * Copyright (c) 2005 - 2011 Miek Gieben
  * See LICENSE for the license
  */
 
 #include "rdup.h"
 
-extern sig_atomic_t sig;
+extern int sig;
 
 void
 got_sig(int signal)
@@ -19,6 +19,7 @@ got_sig(int signal)
 void
 signal_abort(int signal)
 {
+	int status;
         switch(signal) {
                 case SIGPIPE:
                         msg(_("SIGPIPE received, exiting"));
@@ -26,6 +27,12 @@ signal_abort(int signal)
                 case SIGINT:
                         msg(_("SIGINT received, exiting"));
                         break;
+		case SIGCHLD:
+			(void)wait(&status);
+#ifdef DEBUG
+			msg("Wait stat: %d\n", status);
+#endif /* DEBUG */
+			return;
                 default:
                         msg(_("Unhandled signal reveived, exiting"));
                         break;
