@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Miek Gieben
+ * Copyright (c) 2009 - 2011 Miek Gieben
  * See LICENSE for the license
  */
 
@@ -12,23 +12,16 @@ usage_tr(FILE *f)
 {
         fprintf(f, _("USAGE: %s [OPTION]... \n"), PROGNAME);
 	fputs( _("\
-Translate rdup output to something else	and optionally filter it\n\
-through other processes. The input must be rdup's default ouput\n\
-format: \'%p%T %b %u %g %l %s %n\\n\'.\n\
-The output is equal to rdup -c ouput.\n\
+Translate rdup output into something else.\n\
 \n\
 \n\
     OPTIONS:\n\
         -c\t\tforce output to tty\n\
-        -P CMD,opt0,...,opt6\n\
-	    \t\tfilter through CMD\n\
-	\t\tthis may be repeated, output will be filtered\n\
-	\t\tthrough all commands\n\
 	-X FILE\t\tencrypt all paths with AES and the key from FILE\n\
 	-Y FILE\t\tdecrypt all paths with AES and the key from FILE\n\
 	-h\t\tthis help\n\
 	-V\t\tprint version\n\
-        -O FMT\t\toutput format: pax, cpio, tar or rdup\n\
+        -O FMT\t\toutput format: pax, cpio, tar or rdup* (* = default)\n\
 	\t\trdup uses format: \"%p%T %b %u %g %l %s\\n%n%C\"\n\
 	-L\t\tset input format to a list of pathnames\n\
 	-v\t\tbe more verbose and print processed files to stderr\n\
