@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Miek Gieben
+ * Copyright (c) 2009 - 2011 Miek Gieben
  * See LICENSE for the license
  */
 
@@ -13,17 +13,17 @@ usage_up(FILE *f)
         fprintf(f, _("USAGE: %s [OPTION]... DIRECTORY\n"), PROGNAME);
 	fputs( _("\
 Update a directory tree with an rdup archive.\n\
-The input must be rdup -c output\n\
 \n\
         DIRECTORY\twhere to unpack the archive\n\
 \n\
 \n\
     OPTIONS:\n\
-	-t\t\tcreate DIRECTORY if is does not exist\n\
+	-t\t\tcreate DIRECTORY if it does not exist\n\
 	-s NUM\t\tstrip NUM leading path components\n\
 	-r PATH\t\tstrip PATH from each pathname\n\
 	-n\t\tdry run, do not touch the filesystem\n\
 	-V\t\tprint version\n\
+	-T\t\tshow table of contents. DIRECTORY is optional\n\
 	-h\t\tthis help\n\
 	-v\t\tbe more verbose and print processed files to stdout\n\
 \n\
