@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 - 2008 Miek Gieben
+ * Copyright (c) 2005 - 2011 Miek Gieben
  * See LICENSE for the license
  */
 
@@ -10,7 +10,7 @@ extern char *PROGNAME;
 void
 usage(FILE *f)
 {
-        fprintf(f, _("USAGE: %s [OPTION]... FILELIST [ DIR|FILE ]...\n"), PROGNAME);
+        fprintf(f, _("USAGE: %s [OPTION]... FILELIST  [ DIR | FILE ]...\n"), PROGNAME);
 	fputs( _("\
 Generate a full or incremental file list. This list can be used to\n\
 implement a (incremental) backup scheme.\n\
@@ -27,10 +27,13 @@ implement a (incremental) backup scheme.\n\
         \t\tdefaults to: \"%p%T %b %u %g %l %s %n\\n\"\n\
 	-R\t\treverse the output (depth first, first the dirs then the files)\n\
 	-E FILE\t\tuse FILE as an exclude list\n\
-        -0\t\tdelimit internal filelist with NULLs\n\
+	-P CMD,opt0,...,opt6\n\
+	\t\tfilter file contents through CMD\n\
+	\t\tmay be repeated, output will be filtered through all commands\n\
         -V\t\tprint version\n"), f);
 	fputs( _("\
-        -c\t\tcat the contents (FORMAT=\"%p%T %b %u %g %l %s\\n%n%C\")\n\
+        -a\t\treset atime\n\
+        -c\t\tforce output to tty\n\
         -m\t\tonly print new/modified files (unsets -r)\n\
         -n\t\tignore .nobackup files\n\
         -r\t\tonly print removed files (unsets -m)\n\
